@@ -8,6 +8,7 @@ use core::{cmp, fmt, usize};
 mod sealed {
     use super::*;
 
+    #[allow(unreachable_pub)]
     pub trait Sealed {}
     impl Sealed for &str {}
     impl Sealed for char {}
@@ -21,17 +22,16 @@ mod sealed {
     impl<C: MultiCharEq> Sealed for MultiCharEqPattern<C> {}
 }
 
+#[allow(unreachable_pub)]
 pub trait Pattern<'a>: Sized + sealed::Sealed {
     type Searcher: Searcher<'a>;
 
     fn into_searcher(self, haystack: &'a str) -> Self::Searcher;
 
-    #[inline]
     fn is_contained_in(self, haystack: &'a str) -> bool {
         self.into_searcher(haystack).next_match().is_some()
     }
 
-    #[inline]
     fn is_prefix_of(self, haystack: &'a str) -> bool {
         match self.into_searcher(haystack).next() {
             SearchStep::Match(0, _) => true,
@@ -39,18 +39,14 @@ pub trait Pattern<'a>: Sized + sealed::Sealed {
         }
     }
 
-    #[inline]
     fn is_suffix_of(self, haystack: &'a str) -> bool
-    where
-        Self::Searcher: ReverseSearcher<'a>,
-    {
+    where Self::Searcher: ReverseSearcher<'a> {
         match self.into_searcher(haystack).next_back() {
             SearchStep::Match(_, j) if haystack.len() == j => true,
             _ => false,
         }
     }
 
-    #[inline]
     fn strip_prefix_of(self, haystack: &'a str) -> Option<&'a str> {
         if let SearchStep::Match(start, len) = self.into_searcher(haystack).next() {
             debug_assert_eq!(
@@ -63,11 +59,8 @@ pub trait Pattern<'a>: Sized + sealed::Sealed {
         }
     }
 
-    #[inline]
     fn strip_suffix_of(self, haystack: &'a str) -> Option<&'a str>
-    where
-        Self::Searcher: ReverseSearcher<'a>,
-    {
+    where Self::Searcher: ReverseSearcher<'a> {
         if let SearchStep::Match(start, end) = self.into_searcher(haystack).next_back() {
             debug_assert_eq!(
                 end,
@@ -81,6 +74,7 @@ pub trait Pattern<'a>: Sized + sealed::Sealed {
     }
 }
 
+#[allow(unreachable_pub)]
 #[derive(Copy, Clone, Eq, PartialEq, Debug)]
 pub enum SearchStep {
     Match(usize, usize),
@@ -88,11 +82,11 @@ pub enum SearchStep {
     Done,
 }
 
+#[allow(unreachable_pub)]
 pub unsafe trait Searcher<'a> {
     fn haystack(&self) -> &'a str;
     fn next(&mut self) -> SearchStep;
 
-    #[inline]
     fn next_match(&mut self) -> Option<(usize, usize)> {
         loop {
             match self.next() {
@@ -103,7 +97,6 @@ pub unsafe trait Searcher<'a> {
         }
     }
 
-    #[inline]
     fn next_reject(&mut self) -> Option<(usize, usize)> {
         loop {
             match self.next() {
@@ -115,10 +108,10 @@ pub unsafe trait Searcher<'a> {
     }
 }
 
+#[allow(unreachable_pub)]
 pub unsafe trait ReverseSearcher<'a>: Searcher<'a> {
     fn next_back(&mut self) -> SearchStep;
 
-    #[inline]
     fn next_match_back(&mut self) -> Option<(usize, usize)> {
         loop {
             match self.next_back() {
@@ -129,7 +122,6 @@ pub unsafe trait ReverseSearcher<'a>: Searcher<'a> {
         }
     }
 
-    #[inline]
     fn next_reject_back(&mut self) -> Option<(usize, usize)> {
         loop {
             match self.next_back() {
@@ -141,6 +133,7 @@ pub unsafe trait ReverseSearcher<'a>: Searcher<'a> {
     }
 }
 
+#[allow(unreachable_pub)]
 #[derive(Clone, Debug)]
 pub struct CharSearcher<'a> {
     haystack: &'a str,
@@ -152,11 +145,9 @@ pub struct CharSearcher<'a> {
 }
 
 unsafe impl<'a> Searcher<'a> for CharSearcher<'a> {
-    #[inline]
     fn haystack(&self) -> &'a str {
         self.haystack
     }
-    #[inline]
     fn next(&mut self) -> SearchStep {
         let old_finger = self.finger;
         let slice = unsafe { self.haystack.get_unchecked(old_finger..self.finger_back) };
@@ -173,13 +164,9 @@ unsafe impl<'a> Searcher<'a> for CharSearcher<'a> {
             SearchStep::Done
         }
     }
-    #[inline]
     fn next_match(&mut self) -> Option<(usize, usize)> {
         loop {
-            let bytes = self
-                .haystack
-                .as_bytes()
-                .get(self.finger..self.finger_back)?;
+            let bytes = self.haystack.as_bytes().get(self.finger..self.finger_back)?;
             let last_byte = unsafe { *self.utf8_encoded.get_unchecked(self.utf8_size - 1) };
             if let Some(index) = memchr::memchr(last_byte, bytes) {
                 self.finger += index + 1;
@@ -200,7 +187,6 @@ unsafe impl<'a> Searcher<'a> for CharSearcher<'a> {
 }
 
 unsafe impl<'a> ReverseSearcher<'a> for CharSearcher<'a> {
-    #[inline]
     fn next_back(&mut self) -> SearchStep {
         let old_finger = self.finger_back;
         let slice = unsafe { self.haystack.get_unchecked(self.finger..old_finger) };
@@ -217,7 +203,6 @@ unsafe impl<'a> ReverseSearcher<'a> for CharSearcher<'a> {
             SearchStep::Done
         }
     }
-    #[inline]
     fn next_match_back(&mut self) -> Option<(usize, usize)> {
         let haystack = self.haystack.as_bytes();
         loop {
@@ -247,7 +232,6 @@ unsafe impl<'a> ReverseSearcher<'a> for CharSearcher<'a> {
 impl<'a> Pattern<'a> for char {
     type Searcher = CharSearcher<'a>;
 
-    #[inline]
     fn into_searcher(self, haystack: &'a str) -> Self::Searcher {
         let mut utf8_encoded = [0; 4];
         let utf8_size = self.encode_utf8(&mut utf8_encoded).len();
@@ -261,7 +245,6 @@ impl<'a> Pattern<'a> for char {
         }
     }
 
-    #[inline]
     fn is_contained_in(self, haystack: &'a str) -> bool {
         if (self as u32) < 128 {
             haystack.as_bytes().contains(&(self as u8))
@@ -271,29 +254,21 @@ impl<'a> Pattern<'a> for char {
         }
     }
 
-    #[inline]
     fn is_prefix_of(self, haystack: &'a str) -> bool {
         self.encode_utf8(&mut [0u8; 4]).is_prefix_of(haystack)
     }
 
-    #[inline]
     fn strip_prefix_of(self, haystack: &'a str) -> Option<&'a str> {
         self.encode_utf8(&mut [0u8; 4]).strip_prefix_of(haystack)
     }
 
-    #[inline]
     fn is_suffix_of(self, haystack: &'a str) -> bool
-    where
-        Self::Searcher: ReverseSearcher<'a>,
-    {
+    where Self::Searcher: ReverseSearcher<'a> {
         self.encode_utf8(&mut [0u8; 4]).is_suffix_of(haystack)
     }
 
-    #[inline]
     fn strip_suffix_of(self, haystack: &'a str) -> Option<&'a str>
-    where
-        Self::Searcher: ReverseSearcher<'a>,
-    {
+    where Self::Searcher: ReverseSearcher<'a> {
         self.encode_utf8(&mut [0u8; 4]).strip_suffix_of(haystack)
     }
 }
@@ -304,17 +279,14 @@ trait MultiCharEq {
 }
 
 impl<F> MultiCharEq for F
-where
-    F: FnMut(char) -> bool,
+where F: FnMut(char) -> bool
 {
-    #[inline]
     fn matches(&mut self, c: char) -> bool {
         (*self)(c)
     }
 }
 
 impl MultiCharEq for &[char] {
-    #[inline]
     fn matches(&mut self, c: char) -> bool {
         self.iter().any(|&m| m == c)
     }
@@ -332,23 +304,16 @@ struct MultiCharEqSearcher<'a, C: MultiCharEq> {
 impl<'a, C: MultiCharEq> Pattern<'a> for MultiCharEqPattern<C> {
     type Searcher = MultiCharEqSearcher<'a, C>;
 
-    #[inline]
     fn into_searcher(self, haystack: &'a str) -> MultiCharEqSearcher<'a, C> {
-        MultiCharEqSearcher {
-            haystack,
-            char_eq: self.0,
-            char_indices: haystack.char_indices(),
-        }
+        MultiCharEqSearcher { haystack, char_eq: self.0, char_indices: haystack.char_indices() }
     }
 }
 
 unsafe impl<'a, C: MultiCharEq> Searcher<'a> for MultiCharEqSearcher<'a, C> {
-    #[inline]
     fn haystack(&self) -> &'a str {
         self.haystack
     }
 
-    #[inline]
     fn next(&mut self) -> SearchStep {
         let s = &mut self.char_indices;
         let pre_len = s.as_str().len();
@@ -366,7 +331,6 @@ unsafe impl<'a, C: MultiCharEq> Searcher<'a> for MultiCharEqSearcher<'a, C> {
 }
 
 unsafe impl<'a, C: MultiCharEq> ReverseSearcher<'a> for MultiCharEqSearcher<'a, C> {
-    #[inline]
     fn next_back(&mut self) -> SearchStep {
         let s = &mut self.char_indices;
         let pre_len = s.as_str().len();
@@ -388,44 +352,34 @@ macro_rules! pattern_methods {
         type Searcher = $t;
 
         #[allow(clippy::redundant_closure_call)]
-        #[inline]
         fn into_searcher(self, haystack: &'a str) -> $t {
             ($smap)(($pmap)(self).into_searcher(haystack))
         }
 
         #[allow(clippy::redundant_closure_call)]
-        #[inline]
         fn is_contained_in(self, haystack: &'a str) -> bool {
             ($pmap)(self).is_contained_in(haystack)
         }
 
         #[allow(clippy::redundant_closure_call)]
-        #[inline]
         fn is_prefix_of(self, haystack: &'a str) -> bool {
             ($pmap)(self).is_prefix_of(haystack)
         }
 
         #[allow(clippy::redundant_closure_call)]
-        #[inline]
         fn strip_prefix_of(self, haystack: &'a str) -> Option<&'a str> {
             ($pmap)(self).strip_prefix_of(haystack)
         }
 
         #[allow(clippy::redundant_closure_call)]
-        #[inline]
         fn is_suffix_of(self, haystack: &'a str) -> bool
-        where
-            $t: ReverseSearcher<'a>,
-        {
+        where $t: ReverseSearcher<'a> {
             ($pmap)(self).is_suffix_of(haystack)
         }
 
         #[allow(clippy::redundant_closure_call)]
-        #[inline]
         fn strip_suffix_of(self, haystack: &'a str) -> Option<&'a str>
-        where
-            $t: ReverseSearcher<'a>,
-        {
+        where $t: ReverseSearcher<'a> {
             ($pmap)(self).strip_suffix_of(haystack)
         }
     };
@@ -433,39 +387,33 @@ macro_rules! pattern_methods {
 
 macro_rules! searcher_methods {
     (forward) => {
-        #[inline]
         fn haystack(&self) -> &'a str {
             self.0.haystack()
         }
-        #[inline]
         fn next(&mut self) -> SearchStep {
             self.0.next()
         }
-        #[inline]
         fn next_match(&mut self) -> Option<(usize, usize)> {
             self.0.next_match()
         }
-        #[inline]
         fn next_reject(&mut self) -> Option<(usize, usize)> {
             self.0.next_reject()
         }
     };
     (reverse) => {
-        #[inline]
         fn next_back(&mut self) -> SearchStep {
             self.0.next_back()
         }
-        #[inline]
         fn next_match_back(&mut self) -> Option<(usize, usize)> {
             self.0.next_match_back()
         }
-        #[inline]
         fn next_reject_back(&mut self) -> Option<(usize, usize)> {
             self.0.next_reject_back()
         }
     };
 }
 
+#[allow(unreachable_pub)]
 #[derive(Clone, Debug)]
 pub struct CharSliceSearcher<'a, 'b>(<MultiCharEqPattern<&'b [char]> as Pattern<'a>>::Searcher);
 
@@ -478,21 +426,16 @@ unsafe impl<'a, 'b> ReverseSearcher<'a> for CharSliceSearcher<'a, 'b> {
 }
 
 impl<'a, 'b> Pattern<'a> for &'b [char] {
-    pattern_methods!(
-        CharSliceSearcher<'a, 'b>,
-        MultiCharEqPattern,
-        CharSliceSearcher
-    );
+    pattern_methods!(CharSliceSearcher<'a, 'b>, MultiCharEqPattern, CharSliceSearcher);
 }
 
+#[allow(unreachable_pub)]
 #[derive(Clone)]
 pub struct CharPredicateSearcher<'a, F>(<MultiCharEqPattern<F> as Pattern<'a>>::Searcher)
-where
-    F: FnMut(char) -> bool;
+where F: FnMut(char) -> bool;
 
 impl<F> fmt::Debug for CharPredicateSearcher<'_, F>
-where
-    F: FnMut(char) -> bool,
+where F: FnMut(char) -> bool
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("CharPredicateSearcher")
@@ -502,28 +445,21 @@ where
     }
 }
 unsafe impl<'a, F> Searcher<'a> for CharPredicateSearcher<'a, F>
-where
-    F: FnMut(char) -> bool,
+where F: FnMut(char) -> bool
 {
     searcher_methods!(forward);
 }
 
 unsafe impl<'a, F> ReverseSearcher<'a> for CharPredicateSearcher<'a, F>
-where
-    F: FnMut(char) -> bool,
+where F: FnMut(char) -> bool
 {
     searcher_methods!(reverse);
 }
 
 impl<'a, F> Pattern<'a> for F
-where
-    F: FnMut(char) -> bool,
+where F: FnMut(char) -> bool
 {
-    pattern_methods!(
-        CharPredicateSearcher<'a, F>,
-        MultiCharEqPattern,
-        CharPredicateSearcher
-    );
+    pattern_methods!(CharPredicateSearcher<'a, F>, MultiCharEqPattern, CharPredicateSearcher);
 }
 
 impl<'a, 'b, 'c> Pattern<'a> for &'c &'b str {
@@ -533,17 +469,14 @@ impl<'a, 'b, 'c> Pattern<'a> for &'c &'b str {
 impl<'a, 'b> Pattern<'a> for &'b str {
     type Searcher = StrSearcher<'a, 'b>;
 
-    #[inline]
     fn into_searcher(self, haystack: &'a str) -> StrSearcher<'a, 'b> {
         StrSearcher::new(haystack, self)
     }
 
-    #[inline]
     fn is_prefix_of(self, haystack: &'a str) -> bool {
         haystack.as_bytes().starts_with(self.as_bytes())
     }
 
-    #[inline]
     fn strip_prefix_of(self, haystack: &'a str) -> Option<&'a str> {
         if self.is_prefix_of(haystack) {
             unsafe { Some(haystack.get_unchecked(self.as_bytes().len()..)) }
@@ -552,12 +485,10 @@ impl<'a, 'b> Pattern<'a> for &'b str {
         }
     }
 
-    #[inline]
     fn is_suffix_of(self, haystack: &'a str) -> bool {
         haystack.as_bytes().ends_with(self.as_bytes())
     }
 
-    #[inline]
     fn strip_suffix_of(self, haystack: &'a str) -> Option<&'a str> {
         if self.is_suffix_of(haystack) {
             let i = haystack.len() - self.as_bytes().len();
@@ -568,6 +499,7 @@ impl<'a, 'b> Pattern<'a> for &'b str {
     }
 }
 
+#[allow(unreachable_pub)]
 #[derive(Clone, Debug)]
 pub struct StrSearcher<'a, 'b> {
     haystack: &'a str,
@@ -617,12 +549,10 @@ impl<'a, 'b> StrSearcher<'a, 'b> {
 }
 
 unsafe impl<'a, 'b> Searcher<'a> for StrSearcher<'a, 'b> {
-    #[inline]
     fn haystack(&self) -> &'a str {
         self.haystack
     }
 
-    #[inline]
     fn next(&mut self) -> SearchStep {
         match self.searcher {
             StrSearcherImpl::Empty(ref mut searcher) => {
@@ -661,7 +591,6 @@ unsafe impl<'a, 'b> Searcher<'a> for StrSearcher<'a, 'b> {
         }
     }
 
-    #[inline]
     fn next_match(&mut self) -> Option<(usize, usize)> {
         match self.searcher {
             StrSearcherImpl::Empty(..) => loop {
@@ -692,7 +621,6 @@ unsafe impl<'a, 'b> Searcher<'a> for StrSearcher<'a, 'b> {
 }
 
 unsafe impl<'a, 'b> ReverseSearcher<'a> for StrSearcher<'a, 'b> {
-    #[inline]
     fn next_back(&mut self) -> SearchStep {
         match self.searcher {
             StrSearcherImpl::Empty(ref mut searcher) => {
@@ -731,7 +659,6 @@ unsafe impl<'a, 'b> ReverseSearcher<'a> for StrSearcher<'a, 'b> {
         }
     }
 
-    #[inline]
     fn next_match_back(&mut self) -> Option<(usize, usize)> {
         match self.searcher {
             StrSearcherImpl::Empty(..) => loop {
@@ -817,21 +744,16 @@ impl TwoWaySearcher {
         }
     }
 
-    #[inline]
     fn byteset_create(bytes: &[u8]) -> u64 {
         bytes.iter().fold(0, |a, &b| (1 << (b & 0x3f)) | a)
     }
 
-    #[inline]
     fn byteset_contains(&self, byte: u8) -> bool {
         (self.byteset >> ((byte & 0x3f) as usize)) & 1 != 0
     }
 
-    #[inline]
     fn next<S>(&mut self, haystack: &[u8], needle: &[u8], long_period: bool) -> S::Output
-    where
-        S: TwoWayStrategy,
-    {
+    where S: TwoWayStrategy {
         let old_pos = self.position;
         let needle_last = needle.len() - 1;
         'search: loop {
@@ -855,11 +777,8 @@ impl TwoWaySearcher {
                 continue 'search;
             }
 
-            let start = if long_period {
-                self.crit_pos
-            } else {
-                cmp::max(self.crit_pos, self.memory)
-            };
+            let start =
+                if long_period { self.crit_pos } else { cmp::max(self.crit_pos, self.memory) };
             for i in start..needle.len() {
                 if needle[i] != haystack[self.position + i] {
                     self.position += i - self.crit_pos + 1;
@@ -892,11 +811,8 @@ impl TwoWaySearcher {
         }
     }
 
-    #[inline]
     fn next_back<S>(&mut self, haystack: &[u8], needle: &[u8], long_period: bool) -> S::Output
-    where
-        S: TwoWayStrategy,
-    {
+    where S: TwoWayStrategy {
         let old_end = self.end;
         'search: loop {
             let front_byte = match haystack.get(self.end.wrapping_sub(needle.len())) {
@@ -934,11 +850,7 @@ impl TwoWaySearcher {
                 }
             }
 
-            let needle_end = if long_period {
-                needle.len()
-            } else {
-                self.memory_back
-            };
+            let needle_end = if long_period { needle.len() } else { self.memory_back };
             for i in self.crit_pos_back..needle_end {
                 if needle[i] != haystack[self.end - needle.len() + i] {
                     self.end -= self.period;
@@ -959,7 +871,6 @@ impl TwoWaySearcher {
         }
     }
 
-    #[inline]
     fn maximal_suffix(arr: &[u8], order_greater: bool) -> (usize, usize) {
         let mut left = 0;
         let mut right = 1;
@@ -1037,15 +948,12 @@ enum MatchOnly {}
 impl TwoWayStrategy for MatchOnly {
     type Output = Option<(usize, usize)>;
 
-    #[inline]
     fn use_early_reject() -> bool {
         false
     }
-    #[inline]
     fn rejecting(_a: usize, _b: usize) -> Self::Output {
         None
     }
-    #[inline]
     fn matching(a: usize, b: usize) -> Self::Output {
         Some((a, b))
     }
@@ -1056,15 +964,12 @@ enum RejectAndMatch {}
 impl TwoWayStrategy for RejectAndMatch {
     type Output = SearchStep;
 
-    #[inline]
     fn use_early_reject() -> bool {
         true
     }
-    #[inline]
     fn rejecting(a: usize, b: usize) -> Self::Output {
         SearchStep::Reject(a, b)
     }
-    #[inline]
     fn matching(a: usize, b: usize) -> Self::Output {
         SearchStep::Match(a, b)
     }
